@@ -37,20 +37,24 @@ struct xkb_component_names {
     char *symbols;
 };
 
+char *
+text_v1_keymap_get_as_string(struct xkb_keymap *keymap);
+
 XkbFile *
 XkbParseFile(struct xkb_context *ctx, FILE *file,
              const char *file_name, const char *map);
 
 XkbFile *
-XkbParseString(struct xkb_context *ctx, const char *string,
-               const char *file_name);
+XkbParseString(struct xkb_context *ctx,
+               const char *string, size_t len,
+               const char *file_name, const char *map);
 
 void
 FreeXkbFile(XkbFile *file);
 
 XkbFile *
 XkbFileFromComponents(struct xkb_context *ctx,
-                      struct xkb_component_names *kkctgs);
+                      const struct xkb_component_names *kkctgs);
 
 bool
 CompileKeycodes(XkbFile *file, struct xkb_keymap *keymap,
@@ -78,10 +82,10 @@ LookupKeysym(const char *str, xkb_keysym_t *sym_rtrn);
 /***====================================================================***/
 
 static inline bool
-ReportNotArray(struct xkb_keymap *keymap, const char *type, const char *field,
+ReportNotArray(struct xkb_context *ctx, const char *type, const char *field,
                const char *name)
 {
-    log_err(keymap->ctx,
+    log_err(ctx,
             "The %s %s field is not an array; "
             "Ignoring illegal assignment in %s\n",
             type, field, name);
@@ -89,10 +93,10 @@ ReportNotArray(struct xkb_keymap *keymap, const char *type, const char *field,
 }
 
 static inline bool
-ReportShouldBeArray(struct xkb_keymap *keymap, const char *type,
+ReportShouldBeArray(struct xkb_context *ctx, const char *type,
                     const char *field, const char *name)
 {
-    log_err(keymap->ctx,
+    log_err(ctx,
             "Missing subscript for %s %s; "
             "Ignoring illegal assignment in %s\n",
             type, field, name);
@@ -110,10 +114,10 @@ ReportBadType(struct xkb_context *ctx, const char *type, const char *field,
 }
 
 static inline bool
-ReportBadField(struct xkb_keymap *keymap, const char *type, const char *field,
+ReportBadField(struct xkb_context *ctx, const char *type, const char *field,
                const char *name)
 {
-    log_err(keymap->ctx,
+    log_err(ctx,
             "Unknown %s field %s in %s; "
             "Ignoring assignment to unknown field in %s\n",
             type, field, name, name);
