@@ -85,7 +85,8 @@ is_keyboard(int fd)
 }
 
 static int
-keyboard_new(struct dirent *ent, struct xkb_keymap *xkb, struct keyboard **out)
+keyboard_new(struct dirent *ent, struct xkb_keymap *keymap,
+             struct keyboard **out)
 {
     int ret;
     char *path;
@@ -109,7 +110,7 @@ keyboard_new(struct dirent *ent, struct xkb_keymap *xkb, struct keyboard **out)
         goto err_fd;
     }
 
-    state = xkb_state_new(xkb);
+    state = xkb_state_new(keymap);
     if (!state) {
         fprintf(stderr, "Couldn't create xkb state for %s\n", path);
         ret = -EFAULT;
@@ -223,7 +224,7 @@ print_keycode(struct keyboard *kbd, xkb_keycode_t keycode)
     unsigned int nsyms;
     char s[16];
     uint32_t unicode;
-    xkb_layout_index_t group;
+    xkb_layout_index_t layout;
     xkb_mod_index_t mod;
     xkb_led_index_t led;
 
@@ -255,15 +256,12 @@ print_keycode(struct keyboard *kbd, xkb_keycode_t keycode)
     printf("] ");
 #endif
 
-    printf("group [ ");
-    for (group = 0; group < xkb_keymap_num_layouts_for_key(keymap, keycode);
-         group++) {
-        if (xkb_state_layout_index_is_active(state, group,
-                                             XKB_STATE_LAYOUT_EFFECTIVE) <= 0)
-            continue;
-        printf("%s (%d) ", xkb_keymap_layout_get_name(keymap, group), group);
-    }
-    printf("] ");
+    layout = xkb_state_key_get_layout(state, keycode);
+    printf("layout [ %s (%d) ] ",
+           xkb_keymap_layout_get_name(keymap, layout), layout);
+
+    printf("level [ %d ] ",
+           xkb_state_key_get_level(state, keycode, layout));
 
     printf("mods [ ");
     for (mod = 0; mod < xkb_keymap_num_mods(keymap); mod++) {
@@ -499,7 +497,7 @@ main(int argc, char *argv[])
         }
     }
 
-    ctx = test_get_context();
+    ctx = test_get_context(0);
     if (!ctx) {
         ret = -1;
         fprintf(stderr, "Couldn't create xkb context\n");
