@@ -25,111 +25,10 @@
 
 #include "test.h"
 
-enum {
-    DOWN,
-    REPEAT,
-    UP,
-    BOTH,
-    NEXT,
-    FINISH,
-};
-
-#define EVDEV_OFFSET 8
-
-/*
- * Test a sequence of keysyms, resulting from a sequence of key presses,
- * against the keysyms they're supposed to generate.
- *
- * - Each test runs with a clean state.
- * - Each line in the test is made up of:
- *   + A keycode, given as a KEY_* from linux/input.h.
- *   + A direction - DOWN for press, UP for release, BOTH for
- *     immediate press + release, REPEAT to just get the syms.
- *   + A sequence of keysyms that should result from this keypress.
- *
- * The vararg format is:
- * <KEY_*>  <DOWN | UP | BOTH>  <XKB_KEY_* (zero or more)>  <NEXT | FINISH>
- *
- * See below for examples.
- */
-static int
-test_key_seq(struct xkb_keymap *keymap, ...)
-{
-    struct xkb_state *state;
-
-    va_list ap;
-    xkb_keycode_t kc;
-    int op;
-    xkb_keysym_t keysym;
-
-    const xkb_keysym_t *syms;
-    unsigned int nsyms, i;
-    char ksbuf[64];
-
-    state = xkb_state_new(keymap);
-    assert(state);
-
-    va_start(ap, keymap);
-
-    for (;;) {
-        kc = va_arg(ap, int) + EVDEV_OFFSET;
-        op = va_arg(ap, int);
-
-        nsyms = xkb_state_key_get_syms(state, kc, &syms);
-        fprintf(stderr, "got %d syms for key 0x%x: [", nsyms, kc);
-
-        if (op == DOWN || op == BOTH)
-            xkb_state_update_key(state, kc, XKB_KEY_DOWN);
-        if (op == UP || op == BOTH)
-            xkb_state_update_key(state, kc, XKB_KEY_UP);
-
-        for (i = 0; i < nsyms; i++) {
-            keysym = va_arg(ap, int);
-            xkb_keysym_get_name(syms[i], ksbuf, sizeof(ksbuf));
-            fprintf(stderr, "%s%s", (i != 0) ? ", " : "", ksbuf);
-
-            if (keysym == FINISH || keysym == NEXT) {
-                xkb_keysym_get_name(syms[i], ksbuf, sizeof(ksbuf));
-                fprintf(stderr, "Did not expect keysym: %s.\n", ksbuf);
-                goto fail;
-            }
-
-            if (keysym != syms[i]) {
-                xkb_keysym_get_name(keysym, ksbuf, sizeof(ksbuf));
-                fprintf(stderr, "Expected keysym: %s. ", ksbuf);;
-                xkb_keysym_get_name(syms[i], ksbuf, sizeof(ksbuf));
-                fprintf(stderr, "Got keysym: %s.\n", ksbuf);;
-                goto fail;
-            }
-        }
-
-        fprintf(stderr, "]\n");
-
-        keysym = va_arg(ap, int);
-        if (keysym == NEXT)
-            continue;
-        if (keysym == FINISH)
-            break;
-
-        xkb_keysym_get_name(keysym, ksbuf, sizeof(ksbuf));
-        fprintf(stderr, "Expected keysym: %s. Didn't get it.\n", ksbuf);
-        goto fail;
-    }
-
-    va_end(ap);
-    xkb_state_unref(state);
-    return 1;
-
-fail:
-    va_end(ap);
-    xkb_state_unref(state);
-    return 0;
-}
-
 int
 main(void)
 {
-    struct xkb_context *ctx = test_get_context();
+    struct xkb_context *ctx = test_get_context(0);
     struct xkb_keymap *keymap;
 
     assert(ctx);
@@ -204,6 +103,35 @@ main(void)
                         KEY_LEFTSHIFT,   UP,    XKB_KEY_Shift_L,  NEXT,
                         KEY_O,           BOTH,  XKB_KEY_o,        FINISH));
 
+    /*
+     * Two key presses from the same key (e.g. if two keyboards use the
+     * same xkb_state) should only be released after two releases.
+     */
+    assert(test_key_seq(keymap,
+                        KEY_H,           BOTH,  XKB_KEY_h,        NEXT,
+                        KEY_LEFTSHIFT,   DOWN,  XKB_KEY_Shift_L,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_H,        NEXT,
+                        KEY_LEFTSHIFT,   DOWN,  XKB_KEY_Shift_L,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_H,        NEXT,
+                        KEY_LEFTSHIFT,   UP,    XKB_KEY_Shift_L,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_H,        NEXT,
+                        KEY_LEFTSHIFT,   UP,    XKB_KEY_Shift_L,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_h,        FINISH));
+
+    /* Same as above with locked modifiers. */
+    assert(test_key_seq(keymap,
+                        KEY_H,           BOTH,  XKB_KEY_h,          NEXT,
+                        KEY_CAPSLOCK,    DOWN,  XKB_KEY_Caps_Lock,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_H,          NEXT,
+                        KEY_CAPSLOCK,    DOWN,  XKB_KEY_Caps_Lock,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_H,          NEXT,
+                        KEY_CAPSLOCK,    UP,    XKB_KEY_Caps_Lock,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_H,          NEXT,
+                        KEY_CAPSLOCK,    UP,    XKB_KEY_Caps_Lock,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_H,          NEXT,
+                        KEY_CAPSLOCK,    BOTH,  XKB_KEY_Caps_Lock,  NEXT,
+                        KEY_H,           BOTH,  XKB_KEY_h,          FINISH));
+
     /* Group switching / locking. */
     assert(test_key_seq(keymap,
                         KEY_H,        BOTH,  XKB_KEY_h,               NEXT,
@@ -368,12 +296,11 @@ main(void)
 
                         /* Level 5. */
                         KEY_RIGHTALT,    DOWN,  XKB_KEY_ISO_Level5_Shift,  NEXT,
-                        /*
-                         * XXX: This doesn't work, but gives level1 keysyms.
-                         * This does work when when de(neo) is the first layout
-                         * (before us,il etc.). It's like that in the X server
-                         * as well. Investigate.
-                         */
+                        KEY_5,           BOTH,  XKB_KEY_periodcentered,    NEXT,
+                        KEY_E,           BOTH,  XKB_KEY_Up,                NEXT,
+                        KEY_SPACE,       BOTH,  XKB_KEY_KP_0,              NEXT,
+                        KEY_KP8,         BOTH,  XKB_KEY_KP_Up,             NEXT,
+                        KEY_ESC,         BOTH,  XKB_KEY_Escape,            NEXT,
                         KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Shift,  NEXT,
 
                         KEY_V,           BOTH,  XKB_KEY_p,               FINISH));
@@ -418,6 +345,59 @@ main(void)
                         KEY_COMPOSE,   BOTH, XKB_KEY_ISO_Next_Group, NEXT,
                         KEY_H,         BOTH, XKB_KEY_h,              FINISH));
 
+    xkb_keymap_unref(keymap);
+    keymap = test_compile_rules(ctx, "evdev", "", "us,il,ru", "",
+                                "grp:switch,grp:lswitch,grp:menu_toggle");
+    assert(keymap);
+
+    /* Test depressed group works (Mode_switch). */
+    assert(test_key_seq(keymap,
+                        KEY_H,         BOTH, XKB_KEY_h,                 NEXT,
+                        KEY_RIGHTALT,  DOWN, XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_hebrew_yod,        NEXT,
+                        KEY_RIGHTALT,  UP,   XKB_KEY_ISO_Level3_Shift,  NEXT,
+                        KEY_H,         BOTH, XKB_KEY_h,                 NEXT,
+                        KEY_RIGHTALT,  DOWN, XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_hebrew_yod,        NEXT,
+                        KEY_RIGHTALT,  UP,   XKB_KEY_ISO_Level3_Shift,  NEXT,
+                        KEY_H,         BOTH, XKB_KEY_h,                 FINISH));
+
+    /* Test locked+depressed group works, with wrapping and accumulation. */
+    assert(test_key_seq(keymap,
+                        KEY_H,         BOTH, XKB_KEY_h,                 NEXT,
+                        KEY_COMPOSE,   BOTH, XKB_KEY_ISO_Next_Group,    NEXT,
+                        KEY_LEFTALT,   DOWN, XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_Cyrillic_er,       NEXT,
+                        KEY_LEFTALT,   UP,   XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_hebrew_yod,        NEXT,
+                        KEY_COMPOSE,   BOTH, XKB_KEY_ISO_Next_Group,    NEXT,
+                        KEY_LEFTALT,   DOWN, XKB_KEY_Mode_switch,       NEXT,
+                        /* Should wrap back to first group. */
+                        KEY_H,         BOTH, XKB_KEY_h,                 NEXT,
+                        KEY_LEFTALT,   UP,   XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_Cyrillic_er,       NEXT,
+                        KEY_COMPOSE,   BOTH, XKB_KEY_ISO_Next_Group,    NEXT,
+                        KEY_H,         BOTH, XKB_KEY_h,                 NEXT,
+                        /* Two SetGroup(+1)'s should add up. */
+                        KEY_RIGHTALT,  DOWN, XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_hebrew_yod,        NEXT,
+                        KEY_LEFTALT,   DOWN, XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_Cyrillic_er,       NEXT,
+                        KEY_LEFTALT,   UP,   XKB_KEY_Mode_switch,       NEXT,
+                        KEY_H,         BOTH, XKB_KEY_hebrew_yod,        NEXT,
+                        KEY_RIGHTALT,  UP,   XKB_KEY_ISO_Level3_Shift,  NEXT,
+                        KEY_H,         BOTH, XKB_KEY_h,                 FINISH));
+
+    xkb_keymap_unref(keymap);
+    keymap = test_compile_file(ctx, "keymaps/unbound-vmod.xkb");
+    assert(keymap);
+
+    assert(test_key_seq(keymap,
+                        KEY_H,         BOTH, XKB_KEY_h,                 NEXT,
+                        KEY_Z,         BOTH, XKB_KEY_y,                 NEXT,
+                        KEY_MINUS,     BOTH, XKB_KEY_ssharp,            NEXT,
+                        KEY_Z,         BOTH, XKB_KEY_y,                 FINISH));
+
     xkb_keymap_unref(keymap);
     xkb_context_unref(ctx);
     return 0;
