@@ -331,14 +331,41 @@ test_consume(struct xkb_keymap *keymap)
     xkb_state_unref(state);
 }
 
+static void
+key_iter(struct xkb_keymap *keymap, xkb_keycode_t key, void *data)
+{
+    int *counter = (int *) data;
+
+    assert(*counter == key);
+    (*counter)++;
+}
+
+static void
+test_range(struct xkb_keymap *keymap)
+{
+    int counter;
+
+    assert(xkb_keymap_min_keycode(keymap) == 9);
+    assert(xkb_keymap_max_keycode(keymap) == 253);
+
+    counter = xkb_keymap_min_keycode(keymap);
+    xkb_keymap_key_for_each(keymap, key_iter, &counter);
+    assert(counter == xkb_keymap_max_keycode(keymap) + 1);
+}
+
 int
 main(void)
 {
-    struct xkb_context *context = test_get_context();
+    struct xkb_context *context = test_get_context(0);
     struct xkb_keymap *keymap;
 
     assert(context);
 
+    /* Make sure these are allowed. */
+    xkb_context_unref(NULL);
+    xkb_keymap_unref(NULL);
+    xkb_state_unref(NULL);
+
     keymap = test_compile_rules(context, "evdev", "pc104", "us,ru", NULL, "grp:menu_toggle");
     assert(keymap);
 
@@ -346,6 +373,7 @@ main(void)
     test_serialisation(keymap);
     test_repeat(keymap);
     test_consume(keymap);
+    test_range(keymap);
 
     xkb_keymap_unref(keymap);
     xkb_context_unref(context);
