@@ -32,7 +32,7 @@
 int
 main(int argc, char *argv[])
 {
-    struct xkb_context *ctx = test_get_context();
+    struct xkb_context *ctx = test_get_context(0);
     struct xkb_keymap *keymap;
     char *original, *dump;
 
@@ -81,6 +81,16 @@ main(int argc, char *argv[])
     xkb_keymap_unref(keymap);
     keymap = test_compile_string(ctx, dump);
     assert(keymap);
+
+    /* Test response to invalid formats and flags. */
+    assert(!xkb_keymap_new_from_string(ctx, dump, 0, 0));
+    assert(!xkb_keymap_new_from_string(ctx, dump, -1, 0));
+    assert(!xkb_keymap_new_from_string(ctx, dump, XKB_KEYMAP_FORMAT_TEXT_V1+1, 0));
+    assert(!xkb_keymap_new_from_string(ctx, dump, XKB_KEYMAP_FORMAT_TEXT_V1, -1));
+    assert(!xkb_keymap_new_from_string(ctx, dump, XKB_KEYMAP_FORMAT_TEXT_V1, 1414));
+    assert(!xkb_keymap_get_as_string(keymap, 0));
+    assert(!xkb_keymap_get_as_string(keymap, 4893));
+
     xkb_keymap_unref(keymap);
     free(dump);
 
