@@ -103,11 +103,10 @@ extern "C" {
  *
  * The context contains various general library data and state, like
  * logging level and include paths.
+ *
  * Objects are created in a specific context, and multiple contexts may
- * coexist simultaneously. Objects from different contexts are completely
+ * coexist simultaneously.  Objects from different contexts are completely
  * separated and do not share any memory or state.
- * A context is created, accessed, manipulated and destroyed through the
- * xkb_context_*() API.
  */
 struct xkb_context;
 
@@ -120,9 +119,6 @@ struct xkb_context;
  *
  * A keymap is immutable after it is created (besides reference counts, etc.);
  * if you need to change it, you must create a new one.
- *
- * A keymap object is created, accessed and destroyed through the
- * xkb_keymap_*() API.
  */
 struct xkb_keymap;
 
@@ -131,12 +127,9 @@ struct xkb_keymap;
  * Opaque keyboard state object.
  *
  * State objects contain the active state of a keyboard (or keyboards), such
- * as the currently effective layout and the active modifiers. It acts as a
+ * as the currently effective layout and the active modifiers.  It acts as a
  * simple state machine, wherein key presses and releases are the input, and
  * key symbols (keysyms) are the output.
- *
- * A state object is created, accessed, manipulated and destroyed through the
- * xkb_state_*() API.
  */
 struct xkb_state;
 
@@ -155,15 +148,20 @@ struct xkb_state;
  * Keymaps and applications which are compatible with X11 should not use
  * these keycodes.
  *
- * @todo Explain how keycodes are mapped to scancodes.
+ * The values of specific keycodes are determined by the keymap and the
+ * underlying input system.  For example, with an X11-compatible keymap
+ * and Linux evdev scan codes (see linux/input.h), a fixed offset is used:
+ *
+ * @code
+ * xkb_keycode_t keycode_A = KEY_A + 8;
+ * @endcode
  *
  * @sa xkb_keycode_is_legal_ext() xkb_keycode_is_legal_x11()
  */
 typedef uint32_t xkb_keycode_t;
 
 /**
- * A number used to represent the symbols visible on the keycaps of a
- * keyboard.
+ * A number used to represent the symbols generated from a key on a keyboard.
  *
  * A key, represented by a keycode, may generate different symbols according
  * to keyboard state.  For example, on a QWERTY keyboard, pressing the key
@@ -183,7 +181,7 @@ typedef uint32_t xkb_keycode_t;
  *
  * Besides those, any Unicode/ISO 10646 character in the range U0100 to
  * U10FFFF can be represented by a keysym value in the range 0x01000100 to
- * 0x0110FFFF.  The name of Unicode keysyms is "U<codepoint>", e.g. "Ua1b2".
+ * 0x0110FFFF.  The name of Unicode keysyms is "U<codepoint>", e.g. "UA1B2".
  *
  * The name of other unnamed keysyms is the hexadecimal representation of
  * their value, e.g. "0xabcd1234".
@@ -192,32 +190,17 @@ typedef uint32_t xkb_keycode_t;
  */
 typedef uint32_t xkb_keysym_t;
 
-/**
- * Index of a modifier.
- *
- * @todo Explain what are modifiers.
- *
- * Modifier indexes are consecutive.  The first modifier has index 0.
- *
- * Each modifier must have a name, and the names are unique.  Therefore, it
- * is safe to use the name as a unique identifier for a modifier.  Modifier
- * names are case-sensitive.
- *
- * @sa xkb_keymap_num_mods()
- */
-typedef uint32_t xkb_mod_index_t;
-/** A mask of modifier indexes. */
-typedef uint32_t xkb_mod_mask_t;
-
 /**
  * Index of a keyboard layout.
  *
- * @todo Explain what are layouts.
+ * The layout index is a state component which detemines which <em>keyboard
+ * layout</em> is active.  These may be different alphabets, different key
+ * arrangements, etc.
  *
  * Layout indexes are consecutive.  The first layout has index 0.
  *
  * Each layout is not required to have a name, and the names are not
- * guarenteed to be unique (though they are usually provided and unique).
+ * guaranteed to be unique (though they are usually provided and unique).
  * Therefore, it is not safe to use the name as a unique identifier for a
  * layout.  Layout names are case-sensitive.
  *
@@ -236,6 +219,33 @@ typedef uint32_t xkb_layout_mask_t;
  */
 typedef uint32_t xkb_level_index_t;
 
+/**
+ * Index of a modifier.
+ *
+ * A @e modifier is a state component which changes the way keys are
+ * interpreted.  A keymap defines a set of modifiers, such as Alt, Shift,
+ * Num Lock or Meta, and specifies which keys may @e activate which
+ * modifiers (in a many-to-many relationship, i.e. a key can activate
+ * several modifiers, and a modifier may be activated by several keys.
+ * Different keymaps do this differently).
+ *
+ * When retrieving the keysyms for a key, the active modifier set is
+ * consulted; this detemines the correct shift level to use within the
+ * currently active layout (see xkb_level_index_t).
+ *
+ * Modifier indexes are consecutive.  The first modifier has index 0.
+ *
+ * Each modifier must have a name, and the names are unique.  Therefore, it
+ * is safe to use the name as a unique identifier for a modifier.  The names
+ * of some common modifiers are provided in the xkbcommon/xkbcommon-names.h
+ * header file.  Modifier names are case-sensitive.
+ *
+ * @sa xkb_keymap_num_mods()
+ */
+typedef uint32_t xkb_mod_index_t;
+/** A mask of modifier indexes. */
+typedef uint32_t xkb_mod_mask_t;
+
 /**
  * Index of a keyboard LED.
  *
@@ -243,10 +253,10 @@ typedef uint32_t xkb_level_index_t;
  *
  * LED indexes are non-consecutive.  The first LED has index 0.
  *
- * LED names are case-sensitive.
- *
  * Each LED must have a name, and the names are unique. Therefore,
- * it is safe to use the name as a unique identifier for a LED.
+ * it is safe to use the name as a unique identifier for a LED.  The names
+ * of some common LEDs are provided in the xkbcommon/xkbcommon-names.h
+ * header file.  LED names are case-sensitive.
  *
  * @warning A given keymap may specify an exact index for a given LED.
  * Therefore, LED indexing is not necessarily sequential, as opposed to
@@ -263,10 +273,10 @@ typedef uint32_t xkb_led_index_t;
 /** A mask of LED indexes. */
 typedef uint32_t xkb_led_mask_t;
 
-#define XKB_MOD_INVALID     (0xffffffff)
-#define XKB_LAYOUT_INVALID  (0xffffffff)
 #define XKB_KEYCODE_INVALID (0xffffffff)
+#define XKB_LAYOUT_INVALID  (0xffffffff)
 #define XKB_LEVEL_INVALID   (0xffffffff)
+#define XKB_MOD_INVALID     (0xffffffff)
 #define XKB_LED_INVALID     (0xffffffff)
 
 #define XKB_KEYCODE_MAX     (0xffffffff - 1)
@@ -297,13 +307,13 @@ struct xkb_rule_names {
     const char *rules;
     /** The keyboard model by which to interpret keycodes and LEDs. */
     const char *model;
-    /** A comma seperated list of layouts (languages) to include in the
+    /** A comma separated list of layouts (languages) to include in the
      *  keymap. */
     const char *layout;
-    /** A comma seperated list of variants, one per layout, which may
+    /** A comma separated list of variants, one per layout, which may
      *  modify or augment the respective layout in various ways. */
     const char *variant;
-    /** A comma seprated list of options, through which the user specifies
+    /** A comma separated list of options, through which the user specifies
      *  non-layout related preferences, like which key combinations are used
      *  for switching layouts, or which key is the Compose key. */
     const char *options;
@@ -331,7 +341,7 @@ struct xkb_rule_names {
  * @returns The number of bytes in the name, excluding the NUL byte. If
  * the keysym is invalid, returns -1.
  *
- * You may check if truncation has occured by comparing the return value
+ * You may check if truncation has occurred by comparing the return value
  * with the length of buffer, similarly to the snprintf(3) function.
  *
  * @sa xkb_keysym_t
@@ -342,7 +352,7 @@ xkb_keysym_get_name(xkb_keysym_t keysym, char *buffer, size_t size);
 /** Flags for xkb_keysym_from_name(). */
 enum xkb_keysym_flags {
     /** Find keysym by case-insensitive search. */
-    XKB_KEYSYM_CASE_INSENSITIVE = (1 << 0),
+    XKB_KEYSYM_CASE_INSENSITIVE = (1 << 0)
 };
 
 /**
@@ -375,9 +385,9 @@ xkb_keysym_from_name(const char *name, enum xkb_keysym_flags flags);
  * @param[out] buffer A buffer to write the UTF-8 string into.
  * @param[in]  size   The size of buffer.  Must be at least 7.
  *
- * @returns The number of bytes written to the buffer.  If the keysym does
- * not have a Unicode representation, returns 0.  If the buffer is too small,
- * returns -1.
+ * @returns The number of bytes written to the buffer (including the
+ * terminating byte).  If the keysym does not have a Unicode
+ * representation, returns 0.  If the buffer is too small, returns -1.
  */
 int
 xkb_keysym_to_utf8(xkb_keysym_t keysym, char *buffer, size_t size);
@@ -408,6 +418,8 @@ xkb_keysym_to_utf32(xkb_keysym_t keysym);
 enum xkb_context_flags {
     /** Create this context with an empty include path. */
     XKB_CONTEXT_NO_DEFAULT_INCLUDES = (1 << 0),
+    /** Don't take RMLVO names from the environment. */
+    XKB_CONTEXT_NO_ENVIRONMENT_NAMES = (1 << 1),
 };
 
 /**
@@ -439,6 +451,8 @@ xkb_context_ref(struct xkb_context *context);
 /**
  * Release a reference on a context, and possibly free it.
  *
+ * @param context The context.  If it is NULL, this function does nothing.
+ *
  * @memberof xkb_context
  */
 void
@@ -447,7 +461,7 @@ xkb_context_unref(struct xkb_context *context);
 /**
  * Store custom user data in the context.
  *
- * This may be useful in conjuction with xkb_context_set_log_fn() or other
+ * This may be useful in conjunction with xkb_context_set_log_fn() or other
  * callbacks.
  *
  * @memberof xkb_context
@@ -558,7 +572,7 @@ enum xkb_log_level {
     XKB_LOG_LEVEL_ERROR = 20,    /**< Log all errors. */
     XKB_LOG_LEVEL_WARNING = 30,  /**< Log warnings and errors. */
     XKB_LOG_LEVEL_INFO = 40,     /**< Log information, warnings, and errors. */
-    XKB_LOG_LEVEL_DEBUG = 50,    /**< Log everything. */
+    XKB_LOG_LEVEL_DEBUG = 50     /**< Log everything. */
 };
 
 /**
@@ -627,8 +641,8 @@ xkb_context_get_log_verbosity(struct xkb_context *context);
  * function allows you to replace the default behavior with a custom
  * handler.  The handler is only called with messages which match the
  * current logging level and verbosity settings for the context.
- * level is the logging level of the message.  format and args are the
- * same as in the vprintf(3) function.
+ * level is the logging level of the message.  @a format and @a args are
+ * the same as in the vprintf(3) function.
  *
  * You may use xkb_context_set_user_data() on the context, and then call
  * xkb_context_get_user_data() from within the logging function to provide
@@ -654,7 +668,7 @@ xkb_context_set_log_fn(struct xkb_context *context,
 /** Flags for keymap compilation. */
 enum xkb_keymap_compile_flags {
     /** Apparently you can't have empty enums.  What a drag. */
-    XKB_MAP_COMPILE_PLACEHOLDER = 0,
+    XKB_MAP_COMPILE_PLACEHOLDER = 0
 };
 
 /**
@@ -667,7 +681,9 @@ enum xkb_keymap_compile_flags {
  * keymaps.
  *
  * @param context The context in which to create the keymap.
- * @param names   The RMLVO names to use.
+ * @param names   The RMLVO names to use.  In xkbcommon versions prior
+ *                to 0.2.1, this field must be non-NULL.  In later
+ *                versions, passing NULL will use the default keymap.
  * @param flags   Optional flags for the keymap, or 0.
  *
  * @returns A keymap compiled according to the RMLVO names, or NULL if
@@ -681,10 +697,10 @@ xkb_keymap_new_from_names(struct xkb_context *context,
                           const struct xkb_rule_names *names,
                           enum xkb_keymap_compile_flags flags);
 
-/** The possible keymap text formats. */
+/** The possible keymap formats. */
 enum xkb_keymap_format {
     /** The current/classic XKB text format, as generated by xkbcomp -xkb. */
-    XKB_KEYMAP_FORMAT_TEXT_V1 = 1,
+    XKB_KEYMAP_FORMAT_TEXT_V1 = 1
 };
 
 /**
@@ -724,6 +740,20 @@ xkb_keymap_new_from_string(struct xkb_context *context, const char *string,
                            enum xkb_keymap_format format,
                            enum xkb_keymap_compile_flags flags);
 
+/**
+ * Create a keymap from a memory buffer.
+ *
+ * This is just like xkb_keymap_new_from_string(), but takes a length argument
+ * so the input string does not have to be zero-terminated.
+ *
+ * @see xkb_keymap_new_from_string()
+ * @memberof xkb_keymap
+ */
+struct xkb_keymap *
+xkb_keymap_new_from_buffer(struct xkb_context *context, const char *buffer,
+                           size_t length, enum xkb_keymap_format format,
+                           enum xkb_keymap_compile_flags flags);
+
 /**
  * Take a new reference on a keymap.
  *
@@ -737,6 +767,8 @@ xkb_keymap_ref(struct xkb_keymap *keymap);
 /**
  * Release a reference on a keymap, and possibly free it.
  *
+ * @param keymap The keymap.  If it is NULL, this function does nothing.
+ *
  * @memberof xkb_keymap
  */
 void
@@ -779,6 +811,46 @@ xkb_keymap_get_as_string(struct xkb_keymap *keymap,
  * @{
  */
 
+/**
+ * Get the minimum keycode in the keymap.
+ *
+ * @sa xkb_keycode_t
+ * @memberof xkb_keymap
+ */
+xkb_keycode_t
+xkb_keymap_min_keycode(struct xkb_keymap *keymap);
+
+/**
+ * Get the maximum keycode in the keymap.
+ *
+ * @sa xkb_keycode_t
+ * @memberof xkb_keymap
+ */
+xkb_keycode_t
+xkb_keymap_max_keycode(struct xkb_keymap *keymap);
+
+/**
+ * The iterator used by xkb_keymap_key_for_each().
+ *
+ * @sa xkb_keymap_key_for_each
+ * @memberof xkb_keymap
+ */
+typedef void
+(*xkb_keymap_key_iter_t)(struct xkb_keymap *keymap, xkb_keycode_t key,
+                         void *data);
+
+/**
+ * Run a specified function for every valid keycode in the keymap.  If a
+ * keymap is sparse, this function may be called fewer than
+ * (max_keycode - min_keycode + 1) times.
+ *
+ * @sa xkb_keymap_min_keycode() xkb_keymap_max_keycode() xkb_keycode_t
+ * @memberof xkb_keymap
+ */
+void
+xkb_keymap_key_for_each(struct xkb_keymap *keymap, xkb_keymap_key_iter_t iter,
+                        void *data);
+
 /**
  * Get the number of modifiers in the keymap.
  *
@@ -860,6 +932,10 @@ xkb_keymap_num_layouts_for_key(struct xkb_keymap *keymap, xkb_keycode_t key);
 /**
  * Get the number of shift levels for a specific key and layout.
  *
+ * If @c layout is out of range for this key (that is, larger or equal to
+ * the value returned by xkb_keymap_num_layouts_for_key()), it is brought
+ * back into range in a manner consistent with xkb_state_key_get_layout().
+ *
  * @sa xkb_level_index_t
  * @memberof xkb_keymap
  */
@@ -877,19 +953,17 @@ xkb_keymap_num_levels_for_key(struct xkb_keymap *keymap, xkb_keycode_t key,
  *
  * @param[in] keymap    The keymap.
  * @param[in] key       The keycode of the key.
- * @param[in] layout    The layout for which to get the keysyms. This must
- * be smaller than:
- * @code xkb_keymap_num_layouts_for_key(keymap, key) @endcode
- * Usually it would be:
- * @code xkb_state_key_get_layout(state, key) @endcode
+ * @param[in] layout    The layout for which to get the keysyms.
  * @param[in] level     The shift level in the layout for which to get the
  * keysyms. This must be smaller than:
  * @code xkb_keymap_num_layouts_for_key(keymap, key) @endcode
- * usually it would be:
- * @code xkb_state_key_get_level(state, key, layout) @endcode
- * @param[out] syms_out An immutible array of keysyms corresponding the
+ * @param[out] syms_out An immutible array of keysyms corresponding to the
  * key in the given layout and shift level.
  *
+ * If @c layout is out of range for this key (that is, larger or equal to
+ * the value returned by xkb_keymap_num_layouts_for_key()), it is brought
+ * back into range in a manner consistent with xkb_state_key_get_layout().
+ *
  * @returns The number of keysyms in the syms_out array.  If no keysyms
  * are produced by the key in the given layout and shift level, returns 0
  * and sets syms_out to NULL.
@@ -991,6 +1065,8 @@ xkb_state_ref(struct xkb_state *state);
 /**
  * Release a reference on a keybaord state object, and possibly free it.
  *
+ * @param state The state.  If it is NULL, this function does nothing.
+ *
  * @memberof xkb_state
  */
 void
@@ -1014,7 +1090,7 @@ xkb_state_get_keymap(struct xkb_state *state);
 /** Specifies the direction of the key (press / release). */
 enum xkb_key_direction {
     XKB_KEY_UP,   /**< The key was released. */
-    XKB_KEY_DOWN, /**< The key was pressed. */
+    XKB_KEY_DOWN  /**< The key was pressed. */
 };
 
 /**
@@ -1048,7 +1124,7 @@ enum xkb_state_component {
      *  (derived from the other state components). */
     XKB_STATE_LAYOUT_EFFECTIVE = (1 << 7),
     /** LEDs (derived from the other state components). */
-    XKB_STATE_LEDS = (1 << 8),
+    XKB_STATE_LEDS = (1 << 8)
 };
 
 /**
@@ -1074,7 +1150,7 @@ xkb_state_update_key(struct xkb_state *state, xkb_keycode_t key,
  *
  * @param[in]  state    The keyboard state object.
  * @param[in]  key      The keycode of the key.
- * @param[out] syms_out An immutible array of keysyms corresponding the
+ * @param[out] syms_out An immutable array of keysyms corresponding the
  * key in the given keyboard state.
  *
  * As an extension to XKB, this function can return more than one keysym.
@@ -1164,13 +1240,13 @@ enum xkb_state_match {
     XKB_STATE_MATCH_ALL = (1 << 1),
     /** Makes matching non-exclusive, i.e. will not return false if a
      *  modifier not specified in the arguments is active. */
-    XKB_STATE_MATCH_NON_EXCLUSIVE = (1 << 16),
+    XKB_STATE_MATCH_NON_EXCLUSIVE = (1 << 16)
 };
 
 /**
  * Update a keyboard state from a set of explicit masks.
  *
- * This entrypoint is really only for window systems and the like, where a
+ * This entry point is really only for window systems and the like, where a
  * master process holds an xkb_state, then serializes it over a wire
  * protocol, and clients then use the serialization to feed in to their own
  * xkb_state.
@@ -1188,6 +1264,8 @@ enum xkb_state_match {
  * the update.  If nothing in the state has changed, returns 0.
  *
  * @memberof xkb_state
+ *
+ * @sa xkb_state_component
  */
 enum xkb_state_component
 xkb_state_update_mask(struct xkb_state *state,
@@ -1230,7 +1308,7 @@ xkb_state_serialize_mods(struct xkb_state *state,
  * If XKB_STATE_LAYOUT_EFFECTIVE is included, all other state components are
  * ignored.
  *
- * @returns A xkb_layout_mask_t representing the given components of the
+ * @returns A layout index representing the given components of the
  * layout state.
  *
  * This function should not be used in regular clients; please use the
@@ -1324,13 +1402,54 @@ xkb_state_mod_indices_are_active(struct xkb_state *state,
  * Num Lock modifier does not affect this translation at all, even if it
  * active, so it is not consumed by this translation.
  *
- * It may be desireable for some application to not reuse consumed modifiers
- * for further processing, e.g. for hotkeys or keyboard shortcuts.
+ * It may be desirable for some application to not reuse consumed modifiers
+ * for further processing, e.g. for hotkeys or keyboard shortcuts. To
+ * understand why, consider some requirements from a standard shortcut
+ * mechanism, and how they are implemented:
+ *
+ * 1. The shortcut's modifiers must match exactly to the state. For example,
+ *    it is possible to bind separate actions to \<Alt\>\<Tab\> and to
+ *    \<Alt\>\<Shift\>\<Tab\>. Further, if only \<Alt\>\<Tab\> is bound to
+ *    an action, pressing \<Alt\>\<Shift\>\<Tab\> should not trigger the
+ *    shortcut.
+ *    Effectively, this means that the modifiers are compared using the
+ *    equality operator (==).
+ * 2. Only relevant modifiers are considered for the matching. For example,
+ *    Caps Lock and Num Lock should not generally affect the matching, e.g.
+ *    when matching \<Alt\>\<Tab\> against the state, it does not matter
+ *    whether Num Lock is active or not. These relevant, or significant,
+ *    modifiers usually include Alt, Control, Shift, Super and similar.
+ *    Effectively, this means that non-significant modifiers are masked out,
+ *    before doing the comparison as described above.
+ * 3. The matching must be independent of the layout/keymap. For example,
+ *    the \<Plus\> (+) symbol is found on the first level on some layouts,
+ *    and requires holding Shift on others. If you simply bind the action
+ *    to the \<Plus\> keysym, it would work for the unshifted kind, but
+ *    not for the others, because the match against Shift would fail. If
+ *    you bind the action to \<Shift\>\<Plus\>, only the shifted kind would
+ *    work. So what is needed is to recognize that Shift is used up in the
+ *    translation of the keysym itself, and therefore should not be included
+ *    in the matching.
+ *    Effectively, this means that consumed modifiers (Shift in this example)
+ *    are masked out as well, before doing the comparison.
+ *
+ * To summarize, this is how the matching would be performed:
+ * @code
+ *   (keysym == shortcut_keysym) &&
+ *   ((state_modifiers & ~consumed_modifiers & significant_modifiers) == shortcut_modifiers)
+ * @endcode
+ *
+ * @c state_modifiers are the modifiers reported by
+ * xkb_state_mod_index_is_active() and similar functions.
+ * @c consumed_modifiers are the modifiers reported by
+ * xkb_state_mod_index_is_consumed().
+ * @c significant_modifiers are decided upon by the application/toolkit/user;
+ * it is up to them to decide whether these are configurable or hard-coded.
  *
  * @returns 1 if the modifier is consumed, 0 if it is not.  If the modifier
  * index is not valid in the keymap, returns -1.
  *
- * @sa xkb_state_mod_mask_remove_consumend()
+ * @sa xkb_state_mod_mask_remove_consumed()
  * @memberof xkb_state
  */
 int
